#!/usr/local/bin/perl

require 'getopts.pl';

$multifile_flag = 0;
$column_number_flag = 0;
$char_number_flag = 0;
$escs_flag = 0;

&Getopts('cCeh') || die "usage: $0 [-cCeh] regular-expression [filename..]\n";

$column_number_flag = !$column_number_flag if ($opt_c);
if ($opt_C) {
  $char_number_flag = !0;
  undef $/;
}
$escs_flag = !$escs_flag if ($opt_e);

&usage if ($opt_h);

sub usage {
  print <<usage;
LGREP
    lgrep - ɽˤѥޥå


    lgrep [-cCh] regular-expression [ filename ... ]


    lgrep ϡեʸѥ˸
    ΥѥޤԤȥޥå֤򤹤٤ƽϤޤ
    ̾lgrep ϥץ󥹤ˤȿžʸǡ
    ޥå֤Ϥޤ


    -c ѥޤԤΥޥåϰϤͤɽޤ
       ϹԤƬ 0 饫Ȥޤ

    -C ѥޤԤΥޥåʸϰϤͤɽޤ
       ʸϥեƬ 0 饫Ȥޤ

    -e ޥåԤǶĴޤ


     Taiji Yamada (taiji\@aihara.co.jp)
usage
  exit 0;
}

if ($ARGV[0]) {
  $pattern = $ARGV[0];
  shift;
} else {
  &usage;
}

$magic_begin = "\x1a<";
$magic_end   = "\x1a>";

sub check_pattern {
  local($buf) = @_;
  if ($buf =~ /$pattern/) {
    $buf =~ s/$pattern/$magic_begin$&$magic_end/g;
    local($length) = length($buf) + 1;
    print "$ARGV:" if ($multifile_flag &&
                       !$column_number_flag &&
                       !$char_number_flag);
    print "$.:" if ($column_number_flag && !$char_number_flag &&
                    $multifile_flag);
    print "@_" if (!$escs_flag
                   && !$column_number_flag && !$char_number_flag);
    local($c) = 0;
    local($in_flag) = 0;
    local($subbuf) = "";
    for ($i=0; $i<$length-1; $i++) {
      $sub = substr($buf, $i, 2);
      if ($sub eq $magic_begin) {
        $i++;
        $in_flag = !0;
        $subbuf = "";
        if ($column_number_flag) {
          print "$ARGV:" if ($multifile_flag);
          print "$.:$c";
        }
        if ($char_number_flag) {
          print "$ARGV:" if ($multifile_flag);
          print "$c";
        }
        if (!$column_number_flag && !$char_number_flag
            && $escs_flag) {
          print "\x1b[7m";
        }
      } elsif ($sub eq $magic_end) {
        $i++;
        $in_flag = 0;
        print "-$c\n" if ($column_number_flag);
        print "-$c\n" if ($char_number_flag);
        # here, $subbuf is matched string
        if (!$column_number_flag && !$char_number_flag
            && $escs_flag) {
          print "\x1b[0m";
        }
      } else {
        $sub = substr($buf, $i, 1);
        $c++;
        if ($in_flag) {
          $subbuf .= $sub;
        }
        if (!$column_number_flag && !$char_number_flag) {
          if ($sub eq "\n") {
            print "\n";
          } elsif ($in_flag) {
            if (!$escs_flag) {
              print "-";
            } else {
              print "$sub";
            }
          } else {
            if (!$escs_flag) {
              print " ";
            } else {
              print "$sub";
            }
          }
        }
      }
    }
  }
}

$multifile_flag = !0 if ($#ARGV > 0);

while (<>) {
  &check_pattern($_);
  if (eof) {
    close(ARGV);
  }
}
